@@ -52,7 +52,7 @@ class JDownfolder():
 
 class JDownfolder_pickle():
     def __init__(self, inpath, metals, ligands, outpath):
-        fname=os.path.join(inpath, 'TB2J.pickle')
+        fname = os.path.join(inpath, 'TB2J.pickle')
         with open(fname, 'rb') as myfile:
             self.obj = pickle.load(myfile)
 
@@ -61,15 +61,14 @@ class JDownfolder_pickle():
         index_spin = self.obj['index_spin']
         ind_atoms = self.obj['ind_atoms']
         Jdict = self.obj['exchange_Jdict']
-        nspin = len([x for x in index_spin if x>=0])
+        nspin = len([x for x in index_spin if x >= 0])
 
+        self.natom = len(self.atoms)
 
-        self.natom=len(self.atoms)
-
-        self.magnetic_elements= metals
-        self.iM=[]
-        self.iL=[]
-        self.ind_mag_atoms=[]
+        self.magnetic_elements = metals
+        self.iM = []
+        self.iL = []
+        self.ind_mag_atoms = []
         for i, sym in enumerate(self.atoms.get_chemical_symbols()):
             if sym in self.magnetic_elements:
                 self.iM.append(index_spin[i])
@@ -77,13 +76,13 @@ class JDownfolder_pickle():
             elif sym in ligands:
                 self.iL.append(index_spin[i])
 
-        self.nM=len(self.iM)
-        self.nL=len(self.iL)
-        self.nsite=self.nM+self.nL
-
+        self.nM = len(self.iM)
+        self.nL = len(self.iL)
+        self.nsite = self.nM+self.nL
 
         # build matrix form of JR
-        JR = defaultdict(lambda: np.zeros((self.nsite, self.nsite), dtype=float))
+        JR = defaultdict(lambda: np.zeros(
+            (self.nsite, self.nsite), dtype=float))
         for key, val in Jdict.items():
             R, i, j = key
             JR[R][i, j] = val
@@ -95,30 +94,29 @@ class JDownfolder_pickle():
         for i, (key, val) in enumerate(JR.items()):
             JR2[i] = val
 
-
         # sum rule
         iR0 = np.argmin(np.linalg.norm(Rlist, axis=1))
         for i in range(self.nsite):
-            sum_JRi=np.sum(np.sum(JR2, axis=0)[i])
+            sum_JRi = np.sum(np.sum(JR2, axis=0)[i])
             JR2[iR0][i, i] -= sum_JRi
 
         d = JDownfolder(JR2, Rlist, iM=self.iM, iL=self.iL, qmesh=[7, 7, 7])
         Jd = d.get_JR()
 
-        self.Rlist=Rlist
-        self.Rcut=None
+        self.Rlist = Rlist
+        self.Rcut = None
 
         self._prepare_distance()
         self._prepare_index_spin()
-        self.Jdict={}
+        self.Jdict = {}
         for iR, R in enumerate(d.Rlist):
             for i, ispin in enumerate(self.index_spin):
                 for j, jspin in enumerate(self.index_spin):
-                    if ispin>=0 and jspin>=0:
-                        if not (tuple(R) ==(0,0,0) and ispin==jspin): 
+                    if ispin >= 0 and jspin >= 0:
+                        if not (tuple(R) == (0, 0, 0) and ispin == jspin):
                             # self interaction.
-                            self.Jdict[(tuple(R), ispin, jspin)]=Jd[iR, ispin, jspin]
-
+                            self.Jdict[(tuple(R), ispin, jspin)
+                                       ] = Jd[iR, ispin, jspin]
 
         io = SpinIO(atoms=atoms,
                     spinat=self.obj['spinat'],
@@ -163,14 +161,13 @@ class JDownfolder_pickle():
                 self.index_spin.append(-1)
 
 
-
-
 def test(inpath, outpath, metals, ligands):
     #path = "/home/hexu/projects/NiCl2/vasp_inputs/TB2J_results"
     path = "/home/hexu/projects/NiCl2/AFM/TB2J_results"
     #fname = os.path.join(path, "TB2J.pickle")
-    p=JDownfolder_pickle(inpath=inpath, metals=['Ni'], ligands=['Cl'], outpath='TB2J_results_downfolded')
+    p = JDownfolder_pickle(inpath=inpath, metals=['Ni'], ligands=[
+                           'Cl'], outpath='TB2J_results_downfolded')
 
 
-if __name__=="__main__":
+if __name__ == "__main__":
     test()
