@@ -60,7 +60,7 @@ class ExchangeCL2(ExchangeCL):
             # self.S0=self.Gup.S0
 
         self.exchange_Jdict = {}
-        self.exchange_Jdict_orb = {}
+        self.Jiso_orb = {}
 
         self.biquadratic = False
 
@@ -182,19 +182,9 @@ class ExchangeCL2(ExchangeCL):
                 np.dot(self.spinat[iatom], self.spinat[jatom]))
             if is_nonself:
                 self.exchange_Jdict[keyspin] = Jij
-                self.exchange_Jdict_orb[
+                self.Jiso_orb[
                     keyspin] = self.simplify_orbital_contributions(Jorbij, iatom, jatom)
 
-    def simplify_orbital_contributions(self, Jorbij, iatom, jatom):
-        """
-        sum up the contribution of all the orbitals with same (n,l,m)
-        """
-        if self.backend_name == 'SIESTA':
-            mmat_i = self.mmats[iatom]
-            mmat_j = self.mmats[jatom]
-            Jorbij = mmat_i.T @ Jorbij @ mmat_j
-        return Jorbij
-
     def get_rho_e(self, rho_up, rho_dn):
         #self.rho_up_list.append(-1.0 / np.pi * np.imag(rho_up[(0,0,0)]))
         #self.rho_dn_list.append(-1.0 / np.pi * np.imag(rho_dn[(0,0,0)]))
@@ -290,7 +280,7 @@ class ExchangeCL2(ExchangeCL):
             orbital_names=self.orbital_names,
             distance_dict=self.distance_dict,
             exchange_Jdict=self.exchange_Jdict,
-            exchange_Jdict_orb=self.exchange_Jdict_orb,
+            Jiso_orb=self.Jiso_orb,
             dmi_ddict=None,
             NJT_Jdict=None,
             NJT_ddict=None,
