@@ -8,10 +8,12 @@ from ase.cell import Cell
 import numpy as np
 from TB2J import __version__
 
+
 def write_emin(**kwargs):
-    m=SpinIO.load_pickle("./")
+    m = SpinIO.load_pickle("./")
     m.write_Jq(kmesh=[9, 9, 9], path='./', **kwargs)
 
+
 def plot_magnon_band(fname='exchange.xml',
                      path='./',
                      npoints=301,
@@ -19,8 +21,10 @@ def plot_magnon_band(fname='exchange.xml',
                      kvectors=None,
                      knames=None,
                      figfname="magnon_band.pdf",
+                     supercell_matrix=np.eye(3),
                      Jq=False,
                      kpath_fname='exchange_kpth.txt',
+                     ax=None,
                      **kwargs
                      ):
     m = SpinModel(fname=fname, sc_matrix=None)
@@ -30,7 +34,9 @@ def plot_magnon_band(fname='exchange.xml',
                        npoints=npoints,
                        kpath_fname=kpath_fname,
                        Jq=Jq,
-)
+                       supercell_matrix=supercell_matrix,
+                       ax=ax
+                       )
 
     plt.savefig(figfname)
     if show:
